@@ -5,6 +5,7 @@ axes, ticks, tick labels, and grid lines.
 
 import numpy as np
 from astropy import units as u
+from astropy.extern import six
 
 from matplotlib.ticker import Formatter
 from matplotlib.transforms import Affine2D, ScaledTranslation
@@ -18,7 +19,6 @@ from .axislabels import AxisLabels
 from .grid_paths import get_lon_lat_path, get_gridline_path
 from . import settings
 
-from . import six
 
 __all__ = ['CoordinateHelper']
 
@@ -51,6 +51,8 @@ class CoordinateHelper(object):
         self.ticklabels = TickLabels(self.frame,
                                      transform=None,  # display coordinates
                                      figure=parent_axes.get_figure())
+        self.ticks.display_minor_ticks(False)
+        self.minor_frequency = 5
 
         # Initialize axis labels
         self.axislabels = AxisLabels(self.frame,
@@ -166,6 +168,22 @@ class CoordinateHelper(object):
             raise TypeError("formatter should be a string or a Formatter "
                             "instance")
 
+    def set_separator(self, separator):
+        """
+        Set the separator to use for the angle major tick labels.
+
+        Parameters
+        ----------
+        separator : The separator between numbers in sexagesimal
+        representation. Can be either a string or a tuple.
+        """
+        if not (self._formatter_locator.__class__ == AngleFormatterLocator):
+            raise TypeError("Separator can only be specified for angle coordinates")
+        if isinstance(separator, six.string_types) or isinstance(separator, tuple):
+            self._formatter_locator.sep = separator
+        else:
+            raise TypeError("separator should be a string or a tuple")
+
     def set_format_unit(self, unit):
         """
         Set the unit for the major tick labels.
@@ -180,7 +198,7 @@ class CoordinateHelper(object):
         self._formatter_locator.format_unit = unit
 
     def set_ticks(self, values=None, spacing=None, number=None, size=None,
-                  width=None, color=None, alpha=None):
+                  width=None, color=None, alpha=None, exclude_overlapping=False):
         """
         Set the location and properties of the ticks.
 
@@ -199,6 +217,8 @@ class CoordinateHelper(object):
             The length of the ticks in points
         color : str or tuple
             A valid Matplotlib color for the ticks
+        exclude_overlapping : bool, optional
+            Whether to exclude tick labels that overlap over each other.
         """
 
         if sum([values is None, spacing is None, number is None]) < 2:
@@ -224,6 +244,8 @@ class CoordinateHelper(object):
         if alpha is not None:
             self.ticks.set_alpha(alpha)
 
+        self.ticklabels.set_exclude_overlapping(exclude_overlapping)
+
     def set_ticks_position(self, position):
         """
         Set where ticks should appear
@@ -306,14 +328,15 @@ class CoordinateHelper(object):
     def formatter(self):
         return self._formatter_locator.formatter
 
-    def _draw(self, renderer, bboxes):
+    def _draw(self, renderer, bboxes, ticklabels_bbox):
 
         renderer.open_group('coordinate_axis')
 
         self._update_ticks(renderer)
 
         self.ticks.draw(renderer)
-        self.ticklabels.draw(renderer, bboxes=bboxes)
+        self.ticklabels.draw(renderer, bboxes=bboxes,
+                             ticklabels_bbox=ticklabels_bbox)
 
         if self.grid_lines_kwargs['visible']:
 
@@ -324,29 +347,27 @@ class CoordinateHelper(object):
 
             if self._grid_type == 'lines':
 
+                frame_patch = self.frame.patch
                 for path in self.grid_lines:
                     p = PathPatch(path, **self.grid_lines_kwargs)
-                    p.set_clip_path(self.frame.patch)
+                    p.set_clip_path(frame_patch)
                     p.draw(renderer)
 
             else:
 
-                for line in self.grid.collections:
+                for line in self._grid.collections:
                     line.set(**self.grid_lines_kwargs)
                     line.draw(renderer)
 
         renderer.close_group('coordinate_axis')
 
-    def _draw_axislabels(self, renderer, bboxes):
+    def _draw_axislabels(self, renderer, bboxes, ticklabels_bbox, visible_ticks):
 
         renderer.open_group('axis labels')
 
-        visible_ticks = self.ticklabels.get_visible_axes()
-
-        ticklabels_bbox_list = self.ticklabels.get_ticklabels_bbox_list()
-
         self.axislabels.draw(renderer, bboxes=bboxes,
-            ticklabels_bbox_list=ticklabels_bbox_list, visible_ticks=visible_ticks)
+                             ticklabels_bbox_list=ticklabels_bbox,
+                             visible_ticks=visible_ticks)
 
         renderer.close_group('axis labels')
 
@@ -364,6 +385,9 @@ class CoordinateHelper(object):
 
         # First find the ticks we want to show
         tick_world_coordinates, spacing = self._formatter_locator.locator(*coord_range[self.coord_index])
+        if self.ticks.get_display_minor_ticks():
+            minor_ticks_w_coordinates = self._formatter_locator.minor_locator(spacing, self.get_minor_frequency(), *coord_range[self.coord_index])
+
 
         # We want to allow non-standard rectangular frames, so we just rely on
         # the parent axes to tell us what the bounding frame is.
@@ -371,9 +395,8 @@ class CoordinateHelper(object):
 
         self.ticks.clear()
         self.ticklabels.clear()
-        lblinfo = []
-        lbl_world = []
-
+        self.lblinfo = []
+        self.lbl_world = []
         # Look up parent axes' transform from data to figure coordinates.
         #
         # See:
@@ -437,10 +460,21 @@ class CoordinateHelper(object):
             # since the above can produce pairs such as 359 to 361 or 0.5 to
             # 1.5, both of which would match a tick at 0.75. Otherwise we just
             # check the ticks determined above.
-            tick_world_coordinates_unit = tick_world_coordinates.unit
+            self._compute_ticks(tick_world_coordinates, spine, axis, w1, w2, tick_angle)
+
+            if self.ticks.get_display_minor_ticks():
+                self._compute_ticks(minor_ticks_w_coordinates, spine, axis, w1,
+                                    w2, tick_angle, ticks='minor')
+
+        # format tick labels, add to scene
+        text = self._formatter_locator.formatter(self.lbl_world * tick_world_coordinates.unit, spacing=spacing)
+        for kwargs, txt in zip(self.lblinfo, text):
+            self.ticklabels.add(text=txt, **kwargs)
+
+    def _compute_ticks(self, tick_world_coordinates, spine, axis, w1, w2, tick_angle, ticks='major'):
             tick_world_coordinates_values = tick_world_coordinates.value
             if self.coord_type == 'longitude':
-                tick_world_coordinates_values = np.hstack([tick_world_coordinates_values, 
+                tick_world_coordinates_values = np.hstack([tick_world_coordinates_values,
                                                     tick_world_coordinates_values + 360])
 
             for t in tick_world_coordinates_values:
@@ -471,26 +505,55 @@ class CoordinateHelper(object):
                     else:
                         world = t
 
-                    self.ticks.add(axis=axis,
-                                   pixel=(x_data_i, y_data_i),
-                                   world=world,
-                                   angle=angle_i,
-                                   axis_displacement=imin + frac)
+                    if ticks == 'major':
 
-                    # store information to pass to ticklabels.add
-                    # it's faster to format many ticklabels at once outside
-                    # of the loop
-                    lblinfo.append(dict(axis=axis,
-                                   pixel=(x_pix_i, y_pix_i),
-                                   world=world,
-                                   angle=spine.normal_angle[imin],
-                                   axis_displacement=imin + frac))
-                    lbl_world.append(world)
+                        self.ticks.add(axis=axis,
+                                       pixel=(x_data_i, y_data_i),
+                                       world=world,
+                                       angle=angle_i,
+                                       axis_displacement=imin + frac)
 
-        # format tick labels, add to scene
-        text = self._formatter_locator.formatter(lbl_world  * tick_world_coordinates_unit, spacing=spacing)
-        for kwargs, txt in zip(lblinfo, text):
-            self.ticklabels.add(text=txt, **kwargs)
+                        # store information to pass to ticklabels.add
+                        # it's faster to format many ticklabels at once outside
+                        # of the loop
+                        self.lblinfo.append(dict(axis=axis,
+                                                 pixel=(x_pix_i, y_pix_i),
+                                                 world=world,
+                                                 angle=spine.normal_angle[imin],
+                                                 axis_displacement=imin + frac))
+                        self.lbl_world.append(world)
+
+                    else:
+                        self.ticks.add_minor(minor_axis=axis,
+                                             minor_pixel=(x_data_i, y_data_i),
+                                             minor_world=world,
+                                             minor_angle=angle_i,
+                                             minor_axis_displacement=imin + frac)
+
+    def display_minor_ticks(self, display_minor_ticks):
+        """
+        Display minor ticks for this coordinate.
+
+        Parameters
+        ----------
+        display_minor_ticks : bool
+            Whether or not to display minor ticks.
+        """
+        self.ticks.display_minor_ticks(display_minor_ticks)
+
+    def get_minor_frequency(self):
+        return self.minor_frequency
+
+    def set_minor_frequency(self, frequency):
+        """
+        Set the frequency of minor ticks per major ticks.
+
+        Parameters
+        ----------
+        frequency : int
+            The number of minor ticks per major ticks.
+        """
+        self.minor_frequency = frequency
 
     def _update_grid_lines(self):
 
@@ -507,22 +570,40 @@ class CoordinateHelper(object):
         tick_world_coordinates, spacing = self._formatter_locator.locator(*coord_range[self.coord_index])
         tick_world_coordinates_values = tick_world_coordinates.value
 
-        self.grid_lines = []
-        for w in tick_world_coordinates_values:
-            if self.coord_index == 0:
-                x_world = np.repeat(w, 1000)
-                y_world = np.linspace(coord_range[1][0], coord_range[1][1], 1000)
-            else:
-                x_world = np.linspace(coord_range[0][0], coord_range[0][1], 1000)
-                y_world = np.repeat(w, 1000)
-            xy_world = np.vstack([x_world, y_world]).transpose()
-            self.grid_lines.append(self._get_gridline(xy_world))
+        n_coord = len(tick_world_coordinates_values)
+        n_samples = settings.GRID_SAMPLES
 
-    def _get_gridline(self, xy_world):
+        xy_world = np.zeros((n_samples * n_coord, 2))
+
+        self.grid_lines = []
+        for iw, w in enumerate(tick_world_coordinates_values):
+            subset = slice(iw * n_samples, (iw + 1) * n_samples)
+            if self.coord_index == 0:
+                xy_world[subset, 0] = np.repeat(w, n_samples)
+                xy_world[subset, 1] = np.linspace(coord_range[1][0], coord_range[1][1], n_samples)
+            else:
+                xy_world[subset, 0] = np.linspace(coord_range[0][0], coord_range[0][1], n_samples)
+                xy_world[subset, 1] = np.repeat(w, n_samples)
+
+        # We now convert all the world coordinates to pixel coordinates in a
+        # single go rather than doing this in the gridline to path conversion
+        # to fully benefit from vectorized coordinate transformations.
+
+        # Transform line to pixel coordinates
+        pixel = self.transform.inverted().transform(xy_world)
+
+        # Create round-tripped values for checking
+        xy_world_round = self.transform.transform(pixel)
+
+        for iw in range(n_coord):
+            subset = slice(iw * n_samples, (iw + 1) * n_samples)
+            self.grid_lines.append(self._get_gridline(xy_world[subset], pixel[subset], xy_world_round[subset]))
+
+    def _get_gridline(self, xy_world, pixel, xy_world_round):
         if self.coord_type == 'scalar':
-            return get_gridline_path(self.parent_axes, self.transform, xy_world)
+            return get_gridline_path(xy_world, pixel)
         else:
-            return get_lon_lat_path(self.parent_axes, self.transform, xy_world)
+            return get_lon_lat_path(xy_world, pixel, xy_world_round)
 
     def _update_grid_contour(self):
 
@@ -536,7 +617,7 @@ class CoordinateHelper(object):
         tick_world_coordinates, spacing = self._formatter_locator.locator(*coord_range[self.coord_index])
 
         field = field[self.coord_index]
-        
+
         # tick_world_coordinates is a Quantities array and we only needs its values
         tick_world_coordinates_values = tick_world_coordinates.value
 
@@ -554,4 +635,4 @@ class CoordinateHelper(object):
             field[:-1,1:][reset] = np.nan
             field[1:,1:][reset] = np.nan
 
-        self.grid = self.parent_axes.contour(X, Y, field.transpose(), levels=tick_world_coordinates_values)
+        self._grid = self.parent_axes.contour(X, Y, field.transpose(), levels=tick_world_coordinates_values)
